@@ -5,6 +5,8 @@
 #include <vector>
 #include <assert.h>
 
+#include <iostream.hpp>
+
 #include "asc_ctype.hpp"
 #include "clone_ptr-t.hpp"
 #include "config.hpp"
@@ -29,10 +31,10 @@ namespace aspeller {
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 1
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 2
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 3
-    0, 4, 0, 0, 3, 0, 0, 0, 2, 0, 0, 0, 7, 5, 0, 0, // 4
-    0, 0, 0, 0, 0, 0, 6, 1, 0, 0, 0, 0, 0, 0, 0, 0, // 5
-    0, 4, 0, 0, 3, 0, 0, 0, 2, 0, 0, 0, 7, 5, 0, 0, // 6
-    0, 0, 0, 0, 0, 0, 6, 1, 0, 0, 0, 0, 0, 0, 0, 0, // 7
+    0, 4, 0, 0, 3, 0, 0, 0, 2, 0, 0, 0, 6, 5, 0, 0, // 4
+    0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, // 5
+    0, 4, 0, 0, 3, 0, 0, 0, 2, 0, 0, 0, 6, 5, 0, 0, // 6
+    0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, // 7
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 8
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 9
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // A
@@ -113,16 +115,18 @@ namespace aspeller {
       if (!tmp) tmp = nl_langinfo(CODESET);
       if (is_ascii_enc(tmp)) tmp = 0;
       if (tmp)
-        mesg_conv_.setup(*config, charset_, fix_encoding_str(tmp, buf));
+        RET_ON_ERR(mesg_conv_.setup(*config, charset_, fix_encoding_str(tmp, buf)));
       else 
 #endif
-        mesg_conv_.setup(*config, charset_, data_encoding_);
+        RET_ON_ERR(mesg_conv_.setup(*config, charset_, data_encoding_));
+      // no need to check for errors here since we know charset_ is a
+      // supported encoding
       to_utf8_.setup(*config, charset_, "utf-8");
       from_utf8_.setup(*config, "utf-8", charset_);
     }
-
+    
     Conv iconv;
-    iconv.setup(*config, data_encoding_, charset_);
+    RET_ON_ERR(iconv.setup(*config, data_encoding_, charset_));
 
     DataPair d;
 
@@ -140,48 +144,46 @@ namespace aspeller {
   
     FStream char_data;
     String char_data_name;
-    find_file(char_data_name,dir1,dir2,charset_,".dat");
+    find_file(char_data_name,dir1,dir2,charset_,".cset");
     RET_ON_ERR(char_data.open(char_data_name, "r"));
     
     String temp;
-    char_data.getline(temp);
-    char_data.getline(temp);
+    char * p;
+    do {
+      p = get_nb_line(char_data, temp);
+    } while (*p != '/');
     
     for (unsigned int i = 0; i != 256; ++i) {
-      char_data.getline(temp);
-      char * p = const_cast<char *>(temp.str());
-      if (strtoul(p, &p, 16) != i) 
+      p = get_nb_line(char_data, temp);
+      if (!p || strtoul(p, &p, 16) != i) 
         return make_err(bad_file_format, char_data_name);
       to_uni_[i] = strtol(p, &p, 16);
-      ++p;
+      while (asc_isspace(*p)) ++p;
       char_type_[i] = static_cast<CharType>(TO_CHAR_TYPE[to_uchar(*p++)]);
-      if (char_type_[i] >= Vowel) {
-        CharInfo inf = LETTER;
-        to_upper_[i] = static_cast<char>(strtol(p, &p, 16));
-        inf |= to_uchar(to_upper_[i]) == i ? UPPER : 0;
-        to_lower_[i] = static_cast<char>(strtol(p, &p, 16));
-        inf |= to_uchar(to_lower_[i]) == i ? LOWER : 0;
-        to_title_[i] = static_cast<char>(strtol(p, &p, 16));
-        inf |= to_uchar(to_title_[i]) == i ? TITLE : 0;
-        to_plain_[i] = static_cast<char>(strtol(p, &p, 16));
-        inf |= to_uchar(to_plain_[i]) == i ? PLAIN : 0;
-        char_info_[i] = inf;
-        if (*p != '\0')
-          return make_err(bad_file_format, char_data_name);
-      } else {
-        to_upper_[i] = i;
-        to_lower_[i] = i;
-        to_title_[i] = i;
-        to_plain_[i] = i;
-        char_info_[i] = LOWER | UPPER | TITLE | PLAIN;
-      }
+      while (asc_isspace(*p)) ++p;
+      ++p; // display, ignored for now
+      CharInfo inf = char_type_[i] >= Letter ? LETTER : 0;
+      to_upper_[i] = static_cast<char>(strtol(p, &p, 16));
+      inf |= to_uchar(to_upper_[i]) == i ? UPPER : 0;
+      to_lower_[i] = static_cast<char>(strtol(p, &p, 16));
+      inf |= to_uchar(to_lower_[i]) == i ? LOWER : 0;
+      to_title_[i] = static_cast<char>(strtol(p, &p, 16));
+      inf |= to_uchar(to_title_[i]) == i ? TITLE : 0;
+      to_plain_[i] = static_cast<char>(strtol(p, &p, 16));
+      inf |= to_uchar(to_plain_[i]) == i ? PLAIN : 0;
+      inf |= to_uchar(to_plain_[i]) == 0 ? PLAIN : 0;
+      sl_first_[i] = static_cast<char>(strtol(p, &p, 16));
+      sl_rest_[i]  = static_cast<char>(strtol(p, &p, 16));
+      char_info_[i] = inf;
     }
 
+    to_plain_[0] = 1; // to make things slightly easier
+    to_plain_[1] = 1;
+
     for (unsigned int i = 0; i != 256; ++i) {
       to_stripped_[i] = to_plain_[(unsigned char)to_lower_[i]];
-      to_sl_[i]       = char_type_[i] <= Vowel ? '\0' : to_stripped_[i];
     }
-
+    
     //
     //
     //
@@ -192,14 +194,22 @@ namespace aspeller {
       buf = "lower";
     else
       buf = "stripped";
+    char * clean_is;
     if (buf == "stripped") {
       store_as_ = Stripped;
-      to_clean_ = to_stripped_;
+      clean_is = to_stripped_;
     } else {
       store_as_ = Lower;
-      to_clean_ = to_lower_;
+      clean_is = to_lower_;
     }
 
+    for (int i = 0; i != 256; ++i) {
+      to_clean_[i] = char_type_[i] >= Letter ? clean_is[i] : 0;
+    }
+
+    to_clean_[0x00] = 0x10; // to make things slightly easier
+    to_clean_[0x10] = 0x10;
+
     clean_chars_   = get_clean_chars(*this);
     
     //
@@ -360,12 +370,10 @@ namespace aspeller {
     }
   }
     
-  // FIXME: Bug, returns true when inword is a prefix of word
-  //        ie (going, go)
   bool SensitiveCompare::operator() (const char * word, 
 				     const char * inlist) const
   {
-    // this will fail if word or inlist is empty
+   // this will fail if word or inlist is empty
     assert (*word != '\0' && *inlist != '\0');
     
     // if begin inlist is a begin char then it must match begin word
@@ -423,27 +431,28 @@ namespace aspeller {
       bool all_upper = lang->is_upper(*word);
       ++word, ++inlist;
       while (*word != '\0' && *inlist != '\0') {
-	if (lang->is_alpha(*word)) {
-	  if (!lang->is_upper(*word))
-	    all_upper = false;
-	  if (ignore_accents) {
-	    if (lang->to_plain(*word) != lang->to_plain(*inlist))
-	      case_compatible = false;
-	  } else if (strip_accents) {
-	    if (*word != lang->to_plain(*inlist))
-	      if (lang->to_lower(*word) != lang->to_plain(lang->to_lower(*inlist)))
-		return false;
-	      else // accents match case does not
-		case_compatible = false;
-	  } else {
-	    if (*word != *inlist)
-	      if (lang->to_lower(*word) != lang->to_lower(*inlist))
-		return false;
-	      else // accents match case does not
-		case_compatible = false;
-	  }
-	}
-	++word, ++inlist;
+        if (!lang->is_upper(*word))
+          all_upper = false;
+        if (ignore_accents) {
+          if (lang->to_plain(*word) != lang->to_plain(*inlist))
+            if (lang->to_stripped(*word) != lang->to_stripped(*inlist))
+              return false;
+            else
+              case_compatible = false;
+        } else if (strip_accents) {
+          if (*word != lang->to_plain(*inlist))
+            if (lang->to_lower(*word) != lang->to_plain(lang->to_lower(*inlist)))
+              return false;
+            else // accents match case does not
+              case_compatible = false;
+        } else {
+          if (*word != *inlist)
+            if (lang->to_lower(*word) != lang->to_lower(*inlist))
+              return false;
+            else // accents match case does not
+              case_compatible = false;
+        }
+        ++word, ++inlist;
       }
       //   if word is all upper than the casing of inlist can be anything
       //   otherwise the casing of tail begin and tail inlist must match
@@ -452,41 +461,10 @@ namespace aspeller {
       if (!case_compatible) 
 	return false;
     }
-    if (*inlist != '\0') ++inlist;
-    assert(*inlist == '\0');
-  
-    //   if end   inlist is a end   char then it must match end word
-    if (lang->special(*(inlist-1)).end) {
-      if (*(inlist-1) != *(word-1))
-	return false;
-    }
-    return true;
-  }
 
-  static inline PosibErr<void> invalid_char(ParmString word, ParmString letter, 
-                                            const char * where)
-  {
-    char m[70];
-    switch (where[0]) {
-    case 'b':
-      snprintf(m, 70, 
-               _("The character '%s' may not appear at the beginning of a word."),
-               letter.str());
-      break;
-    case 'm':
-      snprintf(m, 70, 
-               _("The character '%s' may not appear at the middle of a word."),
-               letter.str());
-      break;
-    case 'e':
-      snprintf(m, 70, 
-               _("The character '%s' may not appear at the end of a word."),
-               letter.str());
-      break;
-    default:
-      abort();
-    }
-    return make_err(invalid_word, word, m);
+    if (lang->special(*word).end) ++word;
+
+    return (*word == '\0' && *inlist == '\0');
   }
 
   PosibErr<void> check_if_valid(const Language & l, ParmString word) {
@@ -494,20 +472,35 @@ namespace aspeller {
       return make_err(invalid_word, MsgConv(l)(word), _("Empty string."));
     const char * i = word;
     if (!l.is_alpha(*i)) {
-      if (!l.special(*i).begin)
-	return invalid_char(MsgConv(l)(word), MsgConv(l)(*i), "beg");
-      else if (!l.is_alpha(*(i+1)))
+      if (!l.special(*i).begin) {
+        char m[70];
+        snprintf(m, 70, 
+                 _("The character '%s' may not appear at the beginning of a word."),
+                 MsgConv(l)(*i));
+        return make_err(invalid_word, MsgConv(l)(word), m);
+      } else if (!l.is_alpha(*(i+1))) {
 	return make_err(invalid_word, MsgConv(l)(word), _("Does not contain any letters."));
+      }
     }
     for (;*(i+1) != '\0'; ++i) { 
       if (!l.is_alpha(*i)) {
-	if (!l.special(*i).middle)
-	  return invalid_char(MsgConv(l)(word), MsgConv(l)(*i), "middle");
+	if (!l.special(*i).middle) {
+          char m[70];
+          snprintf(m, 70, 
+                   _("The character '%s' may not appear at the middle of a word."),
+                   MsgConv(l)(*i));
+          return make_err(invalid_word, MsgConv(l)(word), m);
+        }
       }
     }
     if (!l.is_alpha(*i)) {
-      if (!l.special(*i).end)
-	return invalid_char(MsgConv(l)(word), MsgConv(l)(*i), "end");
+      if (!l.special(*i).end) {
+        char m[70];
+        snprintf(m, 70, 
+                 _("The character '%s' may not appear at the end of a word."),
+                 MsgConv(l)(*i));
+        return make_err(invalid_word, MsgConv(l)(word), m);
+      }
     }
     return no_err;
   }
