@@ -287,7 +287,7 @@ namespace {
     return word_lookup.empty();
   }
 
-  static const char * const cur_check_word = "aspell default speller rowl 1.8";
+  static const char * const cur_check_word = "aspell default speller rowl 1.8b";
 
   struct DataHead {
     // all sizes except the last four must to divisible by:
@@ -720,11 +720,11 @@ namespace {
    
     bool use_jump_tables = use_soundslike || config.retrieve_bool("use-jump-tables");
 
-    Conv iconv;
+    ConvEC iconv;
     if (config.have("encoding"))
-      iconv.setup(config, config.retrieve("encoding"), lang.charset());
+      RET_ON_ERR(iconv.setup(config, config.retrieve("encoding"), lang.charset()));
     else
-      iconv.setup(config, lang.data_encoding(), lang.charset());
+      RET_ON_ERR(iconv.setup(config, lang.data_encoding(), lang.charset()));
 
     //CERR << (affix_compress ? "  AFFIX COMPRESS" : "")
     //     << (use_soundslike ? "  USING SOUNDSLIKE" : "") 
@@ -797,7 +797,7 @@ namespace {
 	unsigned int s = strlen(w0);
 	CharVector tstr;
 	tstr.append(w0, s+1);
-	char * w = iconv(tstr.data(), tstr.size());
+	RET_ON_ERR_SET(iconv(tstr.data(), tstr.size()), char *, w);
         s = strlen(w);
 
         char * p0 = strchr(w, '/');
@@ -871,7 +871,7 @@ namespace {
           // the words 
           for (WordAff * p = exp_list; p; p = p->next)
           {
-            //CERR << p->word << '/' << p->af << '\n';
+            //CERR.printf("%s|%s\n", p->word.str, p->aff);
             s = p->word.size;
             const char * w = p->word.str;
             bool dup = false;
@@ -1110,8 +1110,8 @@ namespace {
         if (word_hash->parms().is_nonexistent(value)) continue;
 
         data.write(value->total_size + 3);
-        data.write(value->word_size);
         data.write((byte)lang.get_word_info(value->data));
+        data.write(value->word_size);
 
         final_hash[i] = data.size();
 
@@ -1176,10 +1176,10 @@ namespace aspeller {
   {
     CachePtr<Language> lang;
     PosibErr<Language *> res = new_language(config);
-    if (!res) return res;
+    if (res.has_err()) return res;
     lang.reset(res.data);
     lang->set_lang_defaults(config);
-    create(els,*lang,config);
+    RET_ON_ERR(create(els,*lang,config));
     return no_err;
   }
 }
