@@ -329,8 +329,9 @@ int main (int argc, const char *argv[])
     return 1;
   }
 
-#ifdef USE_LOCALE
-  const char * codeset = nl_langinfo(CODESET);
+  const char * codeset = 0;
+#ifdef HAVE_LANGINFO_CODESET
+  codeset = nl_langinfo(CODESET);
   if (is_ascii_enc(codeset)) codeset = 0;
 #endif
   if (!utf8_opts.empty()) {
@@ -1218,8 +1219,7 @@ public:
     *this = *static_cast<const IstreamVirEnumeration *>(other);
   }
   Value next() {
-    *in >> data;
-    if (!*in) return 0;
+    if (!in->getline(data)) return 0;
     else return data.c_str();
   }
   bool at_end() const {return *in;}
