@@ -12,15 +12,12 @@
    CURSES_INCLUDE_WORKAROUND_1
    CURSES_ONLY
    HAVE_GETCH
-   HAVE_MBLEN
    All these macros need to have a true value and not just be defined
 */
 
-#include "settings.h"
+#include <wchar.h>
 
-#ifdef DEFINE_XOPEN_SOURCE_EXTENDED
-#  define _XOPEN_SOURCE_EXTENDED 1
-#endif
+#include "settings.h"
 
 #ifdef CURSES_NON_POSIX
 #define CURSES_ONLY 1
@@ -30,20 +27,11 @@
 
 #include "asc_ctype.hpp"
 #include "check_funs.hpp"
-#include "convert.hpp"
-#include "config.hpp"
-
-#include "gettext.h"
-
-using namespace acommon;
 
 StackPtr<CheckerString> state;
 const char * last_prompt = 0;
 StackPtr<Choices> word_choices;
 StackPtr<Choices> menu_choices;
-Conv null_conv;
-extern Conv dconv;
-extern Conv uiconv;
 
 #if   POSIX_TERMIOS
 
@@ -62,7 +50,11 @@ extern "C" {int getch();}
 
 #if HAVE_LIBCURSES
 
-#include CURSES_HEADER
+#  if USE_NCURSESW
+#    include <ncursesw/curses.h>
+#  else
+#    include <curses.h>
+#  endif
 
 #if CURSES_INCLUDE_STANDARD
 
@@ -103,13 +95,7 @@ static SCREEN * term;
 
 #endif // HAVE_LIBCURSES
 
-#ifdef HAVE_MBLEN
-#  include <wchar.h>
-#else
-#  define mblen(x,y) (*(x) ? 1 : 0)
-#endif
-
-static void cleanup (void) {
+void cleanup (void) {
 #if   HAVE_LIBCURSES
   if (use_curses) {
     endwin();
@@ -124,9 +110,9 @@ static void cleanup (void) {
 
 #if   HAVE_LIBCURSES
 
-//static void do_nothing(int) {}
+void do_nothing(int) {}
 
-static void layout_screen();
+void layout_screen();
 
 #ifdef CURSES_ONLY
 
@@ -134,7 +120,7 @@ inline void handle_last_signal() {}
 
 #else
 
-static void save_state() {
+void save_state() {
   getyx(choice_w,cur_y,cur_x);
   choice_text_size = COLS;
   choice_text = new char[choice_text_size];
@@ -144,7 +130,7 @@ static void save_state() {
   endwin();
 }
 
-static void restore_state() {
+void restore_state() {
   delscreen(term);
   term = newterm(0,stdout,stdin);
   set_term(term);
@@ -168,35 +154,35 @@ static void restore_state() {
 }
 
 
-static void suspend_handler(int) {
+void suspend_handler(int) {
   last_signal = SIGTSTP;
 }
   
-static void continue_handler(int) {
+void continue_handler(int) {
   restore_state();
   signal(SIGTSTP, suspend_handler);
   signal(SIGCONT,  continue_handler),
     last_signal = 0;
 }
 
-static void resize_handler(int ) {
+void resize_handler(int ) {
   last_signal = SIGWINCH;
 }
   
-static void resize() {
+void resize() {
   save_state();
   restore_state();
   last_signal = 0;
 }
 
-static void suspend() {
+void suspend() {
   save_state();
   signal(SIGTSTP, SIG_DFL);
   raise(SIGTSTP);
   last_signal = 0;
 }
 
-static inline void handle_last_signal() {
+inline void handle_last_signal() {
   switch (last_signal) {
   case SIGWINCH:
     resize();
@@ -210,7 +196,7 @@ static inline void handle_last_signal() {
 
 #endif
 
-static void layout_screen() {
+void layout_screen() {
   text_w = 0;
   menu_w = 0;
   choice_w = 0;
@@ -241,12 +227,7 @@ static void layout_screen() {
 }
 
 #endif
-
 void begin_check() {
-  
-  //
-  //
-  //
 #if   HAVE_LIBCURSES
 #if   CURSES_ONLY
   use_curses=true;
@@ -284,7 +265,7 @@ void begin_check() {
 #endif
 #if   POSIX_TERMIOS || (HAVE_LIBCURSES && !CURSES_ONLY)
   if (!isatty (STDIN_FILENO)) {
-    puts(_("Error: Stdin not a terminal."));
+    puts("Error: Stdin not a terminal.");
     exit (-1);
   }
   
@@ -309,19 +290,9 @@ void begin_check() {
 #endif
 }
 
-#ifdef HAVE_WIDE_CURSES
-#  define IS_KEY(k) (res == KEY_CODE_YES && c == KEY_##k)
-#  define NOT_KEY (res != KEY_CODE_YES)
-#else
-#  define IS_KEY(k) (c == KEY_##k)
-#  define NOT_KEY (c < 256)
-#endif
-
 void get_line(String & line) {
 #if   HAVE_LIBCURSES
   if (use_curses) {
-    // FIXME: This won't work correctly when combing characters are
-    // involved.
     menu_text = ReplMenu;
     display_menu();
     wnoutrefresh(choice_w);
@@ -335,13 +306,7 @@ void get_line(String & line) {
     int end_x = begin_x;
     while (true) {
       handle_last_signal();
-#ifdef HAVE_WIDE_CURSES
-      wint_t wi = 0;
-      int res = wget_wch(choice_w, &wi);
-      c = wi;
-#else
       c = wgetch(choice_w);
-#endif
       if (c == ERR) continue;
       if (c == '\r' || c == '\n' || c == KEY_ENTER) 
 	break;
@@ -351,61 +316,35 @@ void get_line(String & line) {
       }
       int y,x;
       getyx(choice_w,y,x);
-      if ((IS_KEY(LEFT) || c == control('b')) && begin_x < x) {
+      if ((c == KEY_LEFT || c == control('b')) && begin_x < x) {
 	wmove(choice_w, y,x-1);
-      } else if ((IS_KEY(RIGHT) || c == control('f')) && x < end_x) {
+      } else if ((c == KEY_RIGHT || c == control('f')) && x < end_x) {
 	wmove(choice_w, y,x+1);
-      } else if (IS_KEY(HOME) || c == control('a')) {
+      } else if (c == KEY_HOME || c == control('a')) {
 	wmove(choice_w, y, begin_x);
-      } else if (IS_KEY(END)  || c == control('e')) {
+      } else if (c == KEY_END  || c == control('e')) {
 	wmove(choice_w, y, end_x);
-      } else if ((IS_KEY(BACKSPACE) || c == control('h') || c == '\x7F') 
+      } else if ((c == KEY_BACKSPACE || c == control('h') || c == '\x7F') 
 		 && begin_x < x) {
 	wmove(choice_w, y,x-1);
 	wdelch(choice_w);
 	--end_x;
-      } else if (IS_KEY(DC) || c == control('d')) {
+      } else if (c == KEY_DC || c == control('d')) {
 	wdelch(choice_w);
 	--end_x;
-      } else if (IS_KEY(EOL) || c == control('k')) {
+      } else if (c == KEY_EOL || c == control('k')) {
 	wclrtoeol(choice_w);
 	end_x = x;
-      } else if (x < max_x && 32 <= c && c != '\x7F' && NOT_KEY /*c < 256*/) {
-#ifdef HAVE_WIDE_CURSES
-        wchar_t wc = c;
-        wins_nwstr(choice_w, &wc, 1);
-#else
+      } else if (x < max_x && 32 <= c && c != '\x7F' && c < 256) {
 	winsch(choice_w, c);
-#endif
 	wmove(choice_w, y, x+1);
 	++end_x;
       }
       wrefresh(choice_w);
     }
-#ifdef HAVE_WIDE_CURSES
-    Vector<wchar_t> wstr;
-    for (int i = begin_x; i < end_x; ++i) {
-      cchar_t cc;
-      attr_t att;
-      short cp;
-      mvwin_wch(choice_w, 0, i, &cc);
-      size_t s = getcchar(&cc, 0, &att, &cp, 0);
-      wstr.resize(s+1); // +1 to allow room for the null character
-      getcchar(&cc, wstr.data(), &att, &cp, 0);
-      s = wcstombs(0, wstr.data(), 0);
-      if (s != (size_t)-1) {
-        int pos = line.alloc(s); // this always leaves space for the
-                                 // null character
-        wcstombs(line.data(pos), wstr.data(), s);
-      } else {
-        line += '?';
-      }
-    }
-#else
     for (int i = begin_x; i < end_x; ++i) {
       line += mvwinch(choice_w, 0, i) & A_CHARTEXT;
     }
-#endif
     menu_text = StdMenu;
     display_menu();
     doupdate();
@@ -423,13 +362,7 @@ void get_line(String & line) {
     tcsetattr (STDIN_FILENO, TCSANOW, &new_attributes);
 #endif
   }
-  if (uiconv.conv) {
-    line = uiconv(line);
-  }
 }
-
-#undef IS_KEY
-#undef NOT_KEY
     
 void get_choice(int & c) {
 #if   HAVE_LIBCURSES
@@ -440,8 +373,8 @@ void get_choice(int & c) {
       handle_last_signal();
       c0 = wgetch(choice_w);
     } while (c0 == ERR);
-    if (c0 == KEY_BREAK)
-      c0 = control('c');
+    if (c == KEY_BREAK)
+      c = control('c');
     if (1 <= c0 && c0 < 128) {
       c = static_cast<char>(c0);
       waddch(choice_w,c);
@@ -471,7 +404,7 @@ void get_choice(int & c) {
 }
 
 #if   HAVE_LIBCURSES
-static void new_line(int & l, int y, int height) {
+void new_line(int & l, int y, int height) {
   --l;
   if (y == height - 1) {
     wmove(text_w, 0, 0);
@@ -481,7 +414,7 @@ static void new_line(int & l, int y, int height) {
     wmove(text_w,y+1,0);
   }
 }
-static void new_line(int & l, int height) {
+void new_line(int & l, int height) {
   int y,x;
   getyx(text_w,y,x);
   new_line(l,y,height);
@@ -519,39 +452,21 @@ void display_misspelled_word() {
 
     l = -1;
 
-    const char * j = i->begin();
+    const char * j = i->pbegin();
     while (!i.off_end()) {
 
       int y,x;
-      int y0,x0;
-      getyx(text_w, y, x);
-      x0 = x;
-      y0 = y;
+      getyx(text_w,y,x);
 
       wattrset(text_w,A_NORMAL);
       int last_space_pos = 0;
-      const char * last_space = j;
-      int prev_glyph_pos = 0;
-      const char * prev_glyph = j;
+      const char * last_space = i->pbegin();
+      int w = 0;
 
-      // NOTE: Combining characters after a character at the very end of
-      //   a line will cause an unnecessary word wrap.  Unfortunately I
-      //   do not know how to avoid it as they is no portable way to
-      //   find out if the next character will combine with the
-      //   previous.
-
-      // We check that:
-      // - we haven't reached the end of the text
-      // - we haven't reached the end of the line
-      // - curse haven't jumped to the next screen line
-      // - curse haven't reached the end of the screen
-      while (j < i->end()
-             && *j != '\n' 
-             && (y0 == y && x >= x0) 
-             && x0 < width - 1)
+      while (j < i->pend() && w < width && *j != '\n')
       {
         if (asc_isspace(*j)) {
-          last_space_pos = x;
+          last_space_pos = w;
           last_space = j;
         }
         if (j == word_begin) {
@@ -560,42 +475,26 @@ void display_misspelled_word() {
         } else if (j == word_end) {
           wattrset(text_w,A_NORMAL);
         }
-        
-        int len = mblen(j, MB_CUR_MAX);
-        int res = OK;
+        int len = mbrlen(j, MB_CUR_MAX, NULL);
         if (len > 0) {
-          res = waddnstr(text_w, const_cast<char *>(j), len);
+          waddnstr(text_w, j, len);
         } else {
           waddch(text_w, ' ');
           len = 1;
         }
-
         j += len;
-        x0 = x;
-        getyx(text_w, y, x);
-
-        if (x != x0 || res == ERR) {
-          prev_glyph_pos = x0;
-          prev_glyph = j - len;
-        }
-        
+        ++w;
       }
-      y = y0;
-      if (j == i->end() || *j == '\n') {
+      if (j == i->pend() || *j == '\n') {
         ++i;
-        j = i->begin();
+        j = i->pbegin();
       } else {
-        if (width - last_space_pos < width/3) {
+        if (w - last_space_pos < width/3) {
           wmove(text_w, y, last_space_pos);
           wclrtoeol(text_w);
           j = last_space + 1;
-        } else {
-          wmove(text_w, y, prev_glyph_pos);
-          wclrtoeol(text_w);
-          j = prev_glyph;
         }
         wmove(text_w, y, width-1);
-        wattrset(text_w,A_NORMAL);
         waddch(text_w,'\\');
       }
 
@@ -613,27 +512,25 @@ void display_misspelled_word() {
   } else
 #endif
   {
-    int pre  = word_begin - cur_line->begin();
-    int post = cur_line->end() - word_end;
+    int pre  = word_begin - cur_line->pbegin();
+    int post = cur_line->pend() - word_end;
     if (pre)
-      fwrite(cur_line->begin(), pre, 1, stdout);
+      fwrite(cur_line->pbegin(), pre, 1, stdout);
     putchar('*');
     fwrite(word_begin, word_end - word_begin, 1, stdout);
     putchar('*');
     if (post)
       fwrite(word_end, post, 1, stdout);
   }
+
 }
 
-static inline void put (FILE * out, char c) {putc(c, out);}
-static inline void put (FILE * out, const char * s) {fputs(s, out);}
-static inline void put (FILE * out, const char * s, int size) {fwrite(s, size, 1, out);}
-
-static void print_truncate(FILE * out, const char * word, int width) {
+template <class O>
+static void print_truncate(O *out, const char * word, int width) {
   int i;
   int len = 0;
   for (i = 0; i < width-1 && *word; word += len, ++i) {
-    len = mblen(word, MB_CUR_MAX);
+    len = mbrlen(word, MB_CUR_MAX, NULL);
     if (len > 0) {
       put(out, word, len);
     } else {
@@ -654,27 +551,31 @@ static void print_truncate(FILE * out, const char * word, int width) {
     put(out,' ');
 }
 
-static void display_menu(FILE * out, const Choices * choices, int width, 
-                         Conv & conv = null_conv) 
-{
+template <class O>
+static void display_menu(O * out, const Choices * choices, int width) {
   if (width <= 11) return;
   Choices::const_iterator i = choices->begin();
   while (i != choices->end()) {
     put(out,i->choice);
     put(out,") ");
-    print_truncate(out, conv(i->desc), width/2 - 4);
+    print_truncate(out, i->desc, width/2 - 4);
     put(out,' ');
     ++i;
     if (i != choices->end()) {
       put(out,i->choice);
       put(out,") ");
-      print_truncate(out, conv(i->desc), width/2 - 4);
+      print_truncate(out, i->desc, width/2 - 4);
       ++i;
     }
-    putc('\n', out);
+    new_line(out);
   }
 }
 
+static inline void put (FILE * out, char c) {putc(c, out);}
+static inline void put (FILE * out, const char * s) {fputs(s, out);}
+static inline void put (FILE * out, const char * s, int size) {fwrite(s, size, 1, out);}
+static inline void new_line(FILE * out) {putc('\n', out);}
+
 #if   HAVE_LIBCURSES
 
 static inline void put (WINDOW * w, char c) 
@@ -689,57 +590,15 @@ static inline void put (WINDOW * w, const char * c, int size)
 {
   waddnstr(w,const_cast<char *>(c), size);
 }
-
-static void print_truncate(WINDOW * out, const char * word, int width) {
-  int len = 0;
-  int y,x;
-  int y0;
-  getyx(out, y, x);
-  int stop = x + width - 1;
-  while (x <= stop && *word) {
-    len = mblen(word, MB_CUR_MAX);
-    if (len <= 0) len = 1;
-    put(out, word, len);
-    word += len;
-    y0 = y;
-    getyx(out, y, x);
-    if (y != y0) {y = y0; x = stop + 1; break;}
-  }
-  for (; x <= stop; ++x)
-    put(out, ' ');
-  if (x > stop && *word) {
-    wmove(out, y, stop);
-    put(out,'$');
-  }
-}
-
-static void display_menu(WINDOW * out, const Choices * choices, int width,
-                         Conv & conv = null_conv) 
+static inline void new_line(WINDOW * w) 
 {
-  if (width <= 11) return;
-  Choices::const_iterator i = choices->begin();
   int y,x;
-  getyx(out, y, x);
-  while (i != choices->end()) {
-    wclrtoeol(out);
-    put(out,i->choice);
-    put(out,") ");
-    print_truncate(out, conv(i->desc), width/2 - 4);
-    ++i;
-    if (i != choices->end()) {
-      wmove(out, y, width/2);
-      put(out,i->choice);
-      put(out,") ");
-      print_truncate(out, conv(i->desc), width/2 - 4);
-      ++i;
-    }
-    ++y;
-    wmove(out, y, 0);
-  }
-}
+  getyx(w,y,x);
+  wmove(w,y+1,0);
+} 
 
 #endif
-
+  
 void display_menu() {
 #if   HAVE_LIBCURSES
   if (use_curses && menu_w) {
@@ -749,7 +608,7 @@ void display_menu() {
       getmaxyx(menu_w,height,width);
       werase(menu_w);
       wmove(menu_w,0,0);
-      display_menu(menu_w, word_choices.get(), width, dconv);
+      display_menu(menu_w, word_choices.get(), width);
       wmove(menu_w,5,0);
       display_menu(menu_w, menu_choices.get(), width);
       wnoutrefresh(menu_w);
@@ -764,87 +623,35 @@ void display_menu() {
 	const char * desc;
       };
       static MenuLine menu_items[9] = {
-	{0, 
-         /* TRANSLATORS: This is a literal Key.*/
-         N_("Enter"), 
-         "", 
-         N_("Accept Changes")},
-        {0,
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Backspace"), 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Control-H"), 
-         N_("Delete the previous character")},
-	{"kcub1", 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Left"), 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Control-B"), 
-         N_("Move Back one space")},
-	{"kcuf1", 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Right"), 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Control-F"), 
-         N_("Move Forward one space")},
-	{"khome", 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Home"), 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Control-A"), 
-         N_("Move to the beginning of the line")},
-	{"kend" , 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("End"), 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Control-E"), 
-         N_("Move to the end of the line")},
-	{"kdch1", 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Delete"), 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Control-D"), 
-         N_("Delete the next character")},
-	{0, 
-         "", 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Control-K"), 
-         N_("Kill all characters to the EOL")},
-	{0, 
-         "", 
-         /* TRANSLATORS: This is a literal Key. */
-         N_("Control-C"), 
-         N_("Abort This Operation")},
+	{0, "Enter", "", "Accept Changes"} 
+	, {0, "Backspace", "Control-H", "Delete the previous character"}
+	, {"kcub1", "Left", "Control-B", "Move Back 1 space"}
+	, {"kcuf1", "Right", "Control-F", "Move Forward 1 space"}
+	, {"khome", "Home", "Control-A", "Move to the beginning of the line"}
+	, {"kend" , "End",  "Control-E", "Move to the end of the line"}
+	, {"kdch1", "Delete", "Control-D", "Delete the next charcter"}
+	, {0, "", "Control-K", "Kill all characters to the EOL"}
+	, {0, "", "Control-C", "Abort This Operation"}
       };
       scrollok(menu_w,false);
       werase(menu_w);
-      int beg = 0,end = 0;
-      int y,x,x0;
       for (int i = 0; i != 9; ++i) {
-        wmove(menu_w, i, beg);
+	wmove(menu_w, i, 0);
+	int w = width;
+	int fun_key_desc_width = 12;
+	int control_key_desc_width = 12;
+	if (w < fun_key_desc_width) fun_key_desc_width = w;
 	if (menu_items[i].capname == 0 
-            || tigetstr(const_cast<char *>(menu_items[i].capname)) != 0)
-        {
-          getyx(menu_w, y, x0);
-          put(menu_w, gt_(menu_items[i].fun_key));
-          getyx(menu_w, y, x);
-          if (x < beg || y != i) end = width;
-          else if (x > end) end = x;
-        }
-      }
-      beg = end + 2;
-      if (beg < width) for (int i = 0; i != 9; ++i) {
-        wmove(menu_w, i, beg);
-        put(menu_w, gt_(menu_items[i].control_key));
-        getyx(menu_w, y, x);
-        if (x < beg || y != i) end = width;
-        else if (x > end) end = x;
-      }
-      beg = end + 2;
-      int w = width - beg;
-      if (w > 1) for (int i = 0; i != 9; ++i) {
-        wmove(menu_w, i, beg);
-        print_truncate(menu_w, gt_(menu_items[i].desc), w);
+	    || tigetstr(const_cast<char *>(menu_items[i].capname)) != 0) 
+	  print_truncate(menu_w, menu_items[i].fun_key, fun_key_desc_width);
+	else
+	  print_truncate(menu_w, "", fun_key_desc_width);
+	w -= fun_key_desc_width;
+	if (w < control_key_desc_width) control_key_desc_width = w;
+	print_truncate(menu_w, menu_items[i].control_key, 
+		       control_key_desc_width);
+	w -= control_key_desc_width;
+	print_truncate(menu_w, menu_items[i].desc, w);
       }
       wnoutrefresh(menu_w);
     }
@@ -853,7 +660,7 @@ void display_menu() {
   } else 
 #endif
   {
-    display_menu(stdout, word_choices.get(), 80, dconv);
+    display_menu(stdout, word_choices.get(), 80);
     display_menu(stdout, menu_choices.get(), 80);
   }
 }
